<?php

class Lumav_EveryPay_Block_Everypay extends Mage_Payment_Block_Form
{
    protected $_code    = 'lumav_everypay';
    protected $_parentCode = 'lumav_everypay';
    protected $_gateway = 'everypay';
    protected $logFile  = 'everypay.log';
    protected $_isCard  = false;
    protected $methodsData;
    protected $paymentLinkData;

    protected function _construct()
    {
        $this->method = Mage::getModel('everypay/everypay');
        $this->setTemplate('lumav/everypay/form_payment_methods.phtml');
    }

    /**
     * Returns available payment methods
     * @return array Payment methods
     */
    public function getPaymentMethods()
    {
        $collection = new Varien_Data_Collection();
        $methodsData = $this->getMethodsData();
        foreach ($methodsData as $data) {
            $method = new Varien_Object();
            $method->setData($data);
            $method->setMethodId(uniqid());
            $collection->addItem($method);
        }

        return $collection;
    }
    
    protected function getMethodsData()
    {
        if(!isset($this->methodsData)) {
            $this->methodsData = Mage::helper('everypay')->getEverypayMethods($this->_isCard);
        }
        
        return $this->methodsData;
    }
    
    public function getPaymentCountries()
    {
        $paymentCountries = [];
        
        $methodsData = $this->getMethodsData();
        foreach ($methodsData as $methodData) {
            $countryCode = $methodData['country_code'];
            if(!empty($countryCode) && !isset($paymentCountries[$countryCode])) {
                $paymentCountries[$countryCode] = $this->getCountryName($countryCode) ? : $countryCode;
            }
        }
        
        return $paymentCountries;
    }
    
    protected function getCountryName($countryCode)
    {
        $countryModel = Mage::getSingleton('directory/country')->loadByCode($countryCode);
        if($countryModel->getId()) {
            return $countryModel->getName();
        }
        
        return false;
    }
    
    public function getDefaultPaymentCountry()
    {
        $methodsData = $this->getMethodsData();
        foreach ($methodsData as $methodData) {
            if(!empty($methodData['country_code']) && $methodData['source'] == $this->getSelectedMethod()) {
                return $methodData['country_code'];
            }
        }
        
        return false;
    }
    
    public function getSelectedMethod()
    {
        if($method = $this->getMethod()->getInfoInstance()->getAdditionalInformation('lumav_everypay_method')) {
            return $method;
        }
        
        return false;
    }

    protected function getReturnUrl()
    {
        return Mage::getUrl('everypay/everypay/return', array('_nosid' => true));
    }

    protected function getCallbackUrl()
    {
        return Mage::getUrl('everypay/everypay/callback', array('_nosid' => true));
    }

    /**
     * Returns payment gateway URL
     *
     * @return string Gateway URL
     */
    public function getGatewayUrl()
    {
        return Mage::getStoreConfig('payment/' . $this->_parentCode . '/gateway_url');
    }
    
    public function getPaymentLink()
    {
        if(!isset($this->paymentLinkData)) {
            $this->paymentLinkData = $this->getPaymentLinkData();
        }
        
        return $this->paymentLinkData;
    }

    /**
     * Get payment link for iframe
     * @return array          Payment link
     */
    protected function getPaymentLinkData()
    {
        $params      = $this->getFields();
        $tokenId     = $this->getOrder()->getPayment()->getMethodInstance()->getInfoInstance()->getAdditionalInformation('lumav_everypay_use_token');
        if($tokenId && !$this->isValidToken($tokenId)) {
            $tokenId = false;
        }
        
        try {
            $paymentLink = Mage::getSingleton('everypay/api')->getPaymentLink($params, $tokenId);
            return $paymentLink;
        } catch (Exception $e) {
            Mage::log($e->getMessage(), null, $this->logFile);
        }
        
        return false;
    }

    /**
     * Checks if quick redirect is enabled and
     * returns javascript block that redirects user
     * to bank without intermediate page
     *
     * @return outstr Javascript block
     */
    public function getQuickRedirectScript()
    {
        $outstr = '<script type="text/javascript"><!--
            if($("GatewayForm")){$("GatewayForm").submit();}
            //--></script>';
        return $outstr;
    }

    public function getOrder()
    {
        $orderId = Mage::getSingleton('checkout/session')->getLastOrderId();
        return Mage::getModel('sales/order')->load($orderId);
    }
    
    public function getTokens()
    {
        return Mage::getResourceModel('everypay/token_collection')
            ->addFieldToSelect('*')
            ->addFieldToFilter('customer_id', Mage::getSingleton('customer/session')->getCustomer()->getId())
            ->addFieldToFilter('cc_year', array('gteq' => date('Y')))
            ->addFieldToFilter(array('cc_year', 'cc_month'), array(array('gt' => date('Y')), array('gteq' => date('m'))))
        ;
    }
    
    public function getSelectedToken()
    {
        if($tokenId = $this->getMethod()->getInfoInstance()->getAdditionalInformation('lumav_everypay_use_token')) {
            if($this->isValidToken($tokenId)) {
                return $tokenId;
            }
        } else {
            foreach($this->getTokens() as $token) {
                if($token->getIsDefault()) {
                    return $token->getCcToken();
                }
            }
        }
        
        return -1;
    }
    
    protected function isValidToken($tokenId)
    {
        foreach($this->getTokens() as $token) {
            if($token->getCcToken() == $tokenId) {
                return true;
            }
        }
        
        return false;
    }
    
    public function getEnableToken()
    {
        return Mage::getStoreConfig('payment/' . $this->_parentCode . '/enable_token');
    }

    /**
     * Populates and returns array of fields to be submitted
     * to a bank for payment
     *
     * @return Array
     */
    public function getFields()
    {
        $order = $this->getOrder();
        /* @var $order Mage_Sales_Model_Order */
        switch (Mage::app()->getLocale()->getLocaleCode()) {
            case 'et_EE':
                $language = 'ET';
                break;
            case 'ru_RU':
                $language = 'RU';
                break;
            default:
                $language = 'ET';
                break;
        }

        $billing  = $order->getBillingAddress();
        $shipping = $order->getShippingAddress();
        // downloadable products only orders don't have shipping address
        if (!$shipping) {
            $shipping = $billing;
        }

        if ($this->getGatewayUrl() == Lumav_EveryPay_Model_Source_ApiUrl::$urlList['LIVE']) {
            $username = Mage::getStoreConfig('payment/' . $this->_parentCode . '/api_username');
        } else {
            $username = Mage::getStoreConfig('payment/' . $this->_parentCode . '/api_username_test');
        }

        $paymentMethod = $order->getPayment()->getMethodInstance()->getInfoInstance()->getAdditionalInformation('lumav_everypay_method');

        $timestamp = date('c', time());

        $requiredFields = array(
            'api_username'    => $username,
            'account_name'    => Mage::getStoreConfig('payment/' . $this->_parentCode . '/account_name'),
            'email'           => $billing->getEmail(),
            'amount'          => number_format($order->getTotalDue(), 2, '.', ''),
            'timestamp'       => $timestamp,
            'order_reference' => $order->getIncrementId(),
            'customer_ip'     => $_SERVER['REMOTE_ADDR'],
        );

        foreach ($requiredFields as $field => $value) {
            if (empty($value)) {
                Mage::throwException(sprintf('Required field %s is missing!', $field));
            }
        }

        $fields = array(
            'token_agreement'   => 'unscheduled',
            'nonce'             => $this->getNonce(),
            'customer_url'      => $this->getReturnUrl(),
            'callback_url'      => $this->getCallbackUrl(),
            'preffered_country' => $billing->getCountry(),
            'locale'            => $language,
            'hmac_fields'       => '',
            'payment_method'    => $paymentMethod,
            'integration_details' => array(
                'software'    => 'Magento',
                'version'     => Mage::getVersion(),
                'integration' => 'module'
            ),
        );
        
        if($this->getEnableToken()) {
             $fields['request_token'] = true;
        }

        $fields = array_merge($requiredFields, $fields);

        ksort($fields);

        $fields['hmac_fields'] = implode(',', array_keys($fields));

        $fields['hmac'] = $this->signData($this->prepareData($fields));
        if (Mage::getStoreConfig('payment/' . $this->_parentCode . '/connection_type')) {
            $fields['skin_name'] = Mage::getStoreConfig('payment/' . $this->_parentCode . '/skin_name');
        }

        // Mage::log(print_r($fields, 1), null, $this->logFile);

        return $fields;
    }

    protected function getNonce()
    {
        while (1) {
            $nonce = uniqid(true);
            $r     = Mage::getSingleton('core/resource')
                ->getConnection('read_connection')
                ->query('select nonce from ' . Mage::getSingleton('core/resource')->getTableName('everypay_nonce') . ' where nonce=\'' . $nonce . '\'')
                ->fetchAll();
            if (!count($r)) {
                $r = Mage::getSingleton('core/resource')
                    ->getConnection('write_connection')
                    ->insert(Mage::getSingleton('core/resource')->getTableName('everypay_nonce'), array('nonce' => $nonce))
                ;
                break;
            }
        }
        return $nonce;
    }

    /**
     * Prepare data package for signing
     *
     * @param array $fields
     * @return string
     */
    protected function prepareData(array $fields)
    {
        $arr = array();
        foreach ($fields as $k => $v) {
            $arr[] = $k . '=' . $v;
        }
        return implode('&', $arr);
    }

    protected function signData($data)
    {
        if ($this->getGatewayUrl() == Lumav_EveryPay_Model_Source_ApiUrl::$urlList['LIVE']) {
            $secret = Mage::getStoreConfig('payment/' . $this->_parentCode . '/api_secret');
        } else {
            $secret = Mage::getStoreConfig('payment/' . $this->_parentCode . '/api_secret_test');
        }

        return hash_hmac('sha1', $data, $secret);
    }

}
