<?php

class Lumav_EveryPay_Model_Everypay extends Mage_Payment_Model_Method_Abstract
{
    const _VERIFY_SUCCESS = 1; // payment successful
    const _VERIFY_CANCEL  = 2; // payment unsuccessful
    const _VERIFY_CORRUPT = 3; // wrong or corrupt response

    protected $_canAuthorize   = true;
    protected $_isGateway      = true;
    protected $_canUseCheckout = true;
    protected $logFile         = 'everypay.log';
    protected $_code           = 'lumav_everypay';
    protected $_cardCode       = 'lumav_everypay_card';
    protected $_formBlockType  = 'everypay/form';
    private $statuses          = array(
        'settled'   => self::_VERIFY_SUCCESS,
        'authorized'=> self::_VERIFY_SUCCESS,
        'failed'    => self::_VERIFY_CANCEL,
        'cancelled' => self::_VERIFY_CANCEL,
        'waiting_for_3ds_response' => self::_VERIFY_CORRUPT,
    );

    /**
     * Returns payment gateway URL
     *
     * @return string Gateway URL
     */
    public function getGatewayUrl()
    {
        return Mage::getStoreConfig('payment/' . $this->_code . '/gateway_url');
    }

    public function getOrderPlaceRedirectUrl()
    {
        return Mage::getUrl('everypay/everypay/redirect');
    }

    /**
     * Assign data to info model instance
     *
     * @param   mixed $data
     * @return  Mage_Payment_Model_Method_Abstract
     */
    public function assignData($data)
    {
        $this->getInfoInstance()
            ->setAdditionalInformation('lumav_everypay_method', $data->getData('lumav_everypay_method'));
    }

    /**
     * Verifies response from Everypay
     *
     * @param array $params Response sent by bank and to be verified
     *
     * @return int
     */
    public function verify(array $params = array())
    {
        if ($this->getGatewayUrl() == Lumav_EveryPay_Model_Source_ApiUrl::$urlList['LIVE']) {
            $username = Mage::getStoreConfig('payment/' . $this->_code . '/api_username');
            $secret   = Mage::getStoreConfig('payment/' . $this->_code . '/api_secret');
        } else {
            $username = Mage::getStoreConfig('payment/' . $this->_code . '/api_username_test');
            $secret   = Mage::getStoreConfig('payment/' . $this->_code . '/api_secret_test');
        }

        // only in automatic callback message
        if (isset($params['processing_errors'])) {
            $errors = json_decode($params['processing_errors']);
            if (!empty($errors)) {
                $this->log('(Everypay) Errors: ' . print_r($errors, 1), __METHOD__, __LINE__);
            }

        }
//      if (isset($params['processing_warnings']))
        //      {
        //          $warnings = json_decode($params['processing_warnings']);
        //          if (!empty($warnings))
        //              $this->log('(Everypay) Warnings: ' . print_r($warnings, 1), __METHOD__, __LINE__);
        //      }

        if (!isset($params['api_username']) || ($params['api_username'] !== $username)) {
            $this->log('(Everypay) Invalid username', __METHOD__, __LINE__);
            Mage::getSingleton('checkout/session')->addError('Invalid username.');
            return self::_VERIFY_CORRUPT;
        }

        $now = time();
        $timestamp = strtotime($params['payment_created_at']);
        if ($timestamp > $now) {
            $this->log('(Everypay) Response outdated (now: ' . $now . ', age: ' . ($now - $timestamp) . ')', __METHOD__, __LINE__);
            Mage::getSingleton('checkout/session')->addError('Response outdated.');
            return self::_VERIFY_CORRUPT;
        }

        // Reference number doesn't match.
        if ($this->getOrderNumber() != $params['order_reference']) {
            $this->log('(Everypay): Order number doesn\'t match (potential tampering attempt). Expecting: ' . $this->getOrderNumber(), __METHOD__, __LINE__);
            Mage::getSingleton('checkout/session')->addError('Order number error.');
            return self::_VERIFY_CORRUPT;
        }

        if (isset($params['account_id']) && ($params['account_id'] !== Mage::getStoreConfig('payment/' . $this->_code . '/account_id'))) {
            $this->log('(Everypay) Invalid account ID', __METHOD__, __LINE__);
            Mage::getSingleton('checkout/session')->addError('Invalid account.');
            return self::_VERIFY_CORRUPT;
        }

        // return data is all checked and valid, now check order
        $order = Mage::getModel('sales/order')->loadByIncrementId($params['order_reference']);
        /* @var $order Mage_Sales_Model_Order */
        $orderMethod = $order->getPayment()->getMethod();
        if ($orderMethod != $this->_code && $orderMethod != $this->_cardCode) {
            $this->log('(Everypay): Wrong payment method. Order has: ' . $orderMethod, __METHOD__, __LINE__);
            Mage::getSingleton('checkout/session')->addError('Wrong payment method.');
            return self::_VERIFY_CORRUPT;
        }

        return $this->statuses[$params['payment_state']];
    }

    /**
     * This method creates invoice for current order
     */
    public function createInvoice()
    {
        $orderNumber = $this->getOrderNumber();
        $order       = Mage::getModel('sales/order')->loadByIncrementId($orderNumber);

        if (!$this->isLocked($orderNumber)) {
            if ($order->canInvoice()) {

                if ($this->createLock($orderNumber)) {

                    $invoice = $order->prepareInvoice();
                    $invoice->pay()->register();
                    $invoice->save();

                    $order->setStatus(Mage_Sales_Model_Order::STATE_PROCESSING);
                    $order->save();
                    
                    try {
                        $quote = Mage::getModel('sales/quote')->loadByIdWithoutStore($order->getQuoteId());
                        $quote->setIsActive(false)->save();
                    } catch (Exception $e) {
                        $this->log(sprintf('%s(%s): %s', __METHOD__, __LINE__, print_r($e->getMessage(), true)));
                    }

                    /* Release lock file right after creating invoice */
                    $this->releaseLock($orderNumber);

                    /* Send invoice */
                    if (Mage::getStoreConfig('payment/' . $this->_code . '/invoice_confirmation') == '1') {
                        $invoice->sendEmail(true, '');
                    }

                    Mage::register('current_invoice', $invoice);
                }
            } else {
                $this->log('Failed to create invoice for order ' . $orderNumber . '. Reason: invoice already created', __METHOD__, __LINE__);
            }
        } else {
            $this->log('Failed to create invoice for order ' . $orderNumber . '. Reason: order locked', __METHOD__, __LINE__);
        }
    }

    /**
     *
     * @param string $orderNumber
     * @return string
     */
    private function getLockfilePath($orderNumber)
    {
        return Mage::getBaseDir('var') . DS . 'locks' . DS . 'order_' . $orderNumber . '.lock';
    }

    /**
     * Checks if given invoice is locked, i.e if it has
     * a file in var/locks folder
     *
     * @param string $orderNumber
     */
    public function isLocked($orderNumber)
    {
        return file_exists($this->getLockfilePath($orderNumber));
    }

    /**
     * Locks order, i.e creates a lock file
     * in var/locks folder
     * @param string $orderNumber
     */
    public function createLock($orderNumber)
    {
        $path = $this->getLockfilePath($orderNumber);
        if (!touch($this->getLockfilePath($orderNumber))) {
            $this->log('Failed to create lockfile ' . $path, __METHOD__, __LINE__);
            return false;
        }
        $this->log('Created lockfile ' . $path, __METHOD__, __LINE__);
        return true;
    }

    /**
     * Releases lock for order, i.e deletes
     * lock file from var/locks folder
     *
     * @param string $orderNumber
     */
    public function releaseLock($orderNumber)
    {
        $path = $this->getLockfilePath($orderNumber);
        if (!unlink($path)) {
            $this->log('Failed to delete lockfile ' . $path, __METHOD__, __LINE__);
            return false;
        }
        $this->log('Deleted lockfile ' . $path, __METHOD__, __LINE__);
        return true;
    }

    protected function log($t, $m, $l)
    {
        Mage::log(sprintf('%s(%s)@%s: %s', $m, $l, $_SERVER['REMOTE_ADDR'], $t), null, $this->logFile);
    }

}
